@@ -1,8 +1,10 @@
+if(getRversion() >= "2.15.1") utils::globalVariables("LOCATIONS")
+
 pop.aggregate <- function(pop.pred, regions, method=c('independence', 'regional'),
 						name = method,
 						inputs=list(e0F.sim.dir=NULL, e0M.sim.dir='joint_', tfr.sim.dir=NULL), 
 						verbose=FALSE) {
-	data(LOCATIONS)
+	data("LOCATIONS", package='bayesPop')
 	regions <- unique(regions)
 	method <- match.arg(method)
 	if(missing(name)) name <- method
@@ -41,8 +43,22 @@ pop.aggregate.regional <- function(pop.pred, regions, name,
 	for (item in c('POPm0', 'POPf0', 'MXm', 'MXf', 'MIGm', 'MIGf', 'SRB', 'PASFR', 'MIGtype'))
 		inp[[item]] <- NULL
 	aggregated.countries <- list()
+	aggr.obs.data <- list(male=NULL, female=NULL)
 	if(verbose) cat('\nAggregating inputs using regional method.')
+	status.for.gui <- paste('out of', length(regions), 'regions.')
+	gui.options <- list()
+	obs.data <- inp$pop.matrix
+	region.counter <- 0
 	for(id in regions) {
+		if(getOption('bDem.PopAgpred', default=FALSE)) {
+			# This is to unblock the GUI, if the run is invoked from bayesDem
+			# and pass info about its status
+			# In such a case the gtk libraries are already loaded
+			region.counter <- region.counter + 1
+			gui.options$bDem.PopAgpred.status <- paste('finished', region.counter, status.for.gui)
+			unblock.gtk('bDem.PopAgpred', gui.options)
+		}
+
 		countries <- get.countries.for.region(id, pop.pred)
 		if(length(countries)==0) next
 		inipop <- .aggregate.initial.pop(pop.pred, countries, id)
@@ -56,6 +72,21 @@ pop.aggregate.regional <- function(pop.pred, regions, name,
 		inp$PASFR <- rbind(inp$PASFR, .aggregate.pasfr(pop.pred, countries, countries.index, id))
 		inp$MIGtype <- rbind(inp$MIGtype, .aggregate.migtype(pop.pred, countries, countries.index, id))
 		aggregated.countries[[as.character(id)]] <- countries
+		# aggregate pop.matrix
+		for(cidx in 1:length(countries.index)) {
+			country.obs.idx <- grep(paste('^', countries[cidx], '_', sep=''), rownames(obs.data[['male']]), value=FALSE)
+			if(cidx == 1) {
+				aggr.obs.dataM <- obs.data[['male']][country.obs.idx,]
+				aggr.obs.dataF <- obs.data[['female']][country.obs.idx,]
+				rownames(aggr.obs.dataM) <- rownames(aggr.obs.dataF) <- sub(paste(countries[cidx], '_', sep=''), 
+													paste(id, '_', sep=''), rownames(obs.data[['male']][country.obs.idx,]))
+			} else {
+				aggr.obs.dataM <- aggr.obs.dataM + obs.data[['male']][country.obs.idx,]
+				aggr.obs.dataF <- aggr.obs.dataF + obs.data[['female']][country.obs.idx,]
+			}
+		}
+		aggr.obs.data[['male']] <- rbind(aggr.obs.data[['male']], aggr.obs.dataM)
+		aggr.obs.data[['female']] <- rbind(aggr.obs.data[['female']], aggr.obs.dataF)
 	}
 	dir <- if(is.null(inputs$tfr.sim.dir)) pop.pred$inputs$TFRpred$mcmc.set$meta$parent.meta$output.dir else inputs$tfr.sim.dir
 	if(!is.null(dir)) inp$TFRpred <- get.tfr.prediction(dir)
@@ -68,7 +99,10 @@ pop.aggregate.regional <- function(pop.pred, regions, name,
 	aggr.pred <- do.pop.predict(regions, inp=inp, outdir=outdir, nr.traj=pop.pred$nr.traj, ages=pop.pred$ages, verbose=verbose)
 	aggr.pred$aggregation.method <- 'regional'
 	aggr.pred$aggregated.countries <- aggregated.countries
-	return(aggr.pred)
+	aggr.pred$inputs$pop.matrix <- aggr.obs.data
+	bayesPop.prediction <- aggr.pred
+	save(bayesPop.prediction, file=file.path(outdir, 'prediction.rda'))
+	return(bayesPop.prediction)
 }
 
 .aggregate.by.sum <- function(pop.pred, countries, what.names, id) {
@@ -165,21 +199,33 @@ pop.aggregate.regional <- function(pop.pred, regions, name,
 	return(res)
 }
 
+
 pop.aggregate.independence <- function(pop.pred, regions, name, verbose=verbose) {
 	if(verbose) cat('\nAggregating using independence method.')
 	nreg <- length(regions)
 	quantiles.to.keep <- as.numeric(dimnames(pop.pred$quantiles)[[2]])
-	quant <- quantM <- quantF <- array(NA, c(nreg, dim(pop.pred$quantiles)[2:3]), dimnames=dimnames(pop.pred$quantiles))
+	quant <- quantM <- quantF <- array(NA, c(nreg, dim(pop.pred$quantiles)[2:3]), dimnames=c(list(regions), dimnames(pop.pred$quantiles)[2:3]))
 	quantMage <- quantFage <- quantPropMage <- quantPropFage <- array(NA, c(nreg, dim(pop.pred$quantilesMage)[2:4]),
-						dimnames=dimnames(pop.pred$quantilesMage))
+						dimnames=c(list(regions), dimnames(pop.pred$quantilesMage)[2:4]))
 	mean_sd <- mean_sdM <- mean_sdF <- array(NA, c(nreg,dim(pop.pred$traj.mean.sd)[2:3]))
+	obs.data <- pop.pred$inputs$pop.matrix
+	aggr.obs.data <- list(male=NULL, female=NULL)
 	outdir <- gsub('predictions', paste('aggregations', name, sep='_'), pop.pred$output.directory)
 	if(file.exists(outdir)) unlink(outdir, recursive=TRUE)
 	dir.create(outdir, recursive=TRUE)
 	aggregated.countries <- list()
 	id.idx <- 0
 	valid.regions <- rep(FALSE, length(regions))
+	status.for.gui <- paste('out of', nreg, 'regions.')
+	gui.options <- list()
 	for(reg.idx in 1:length(regions)) {
+		if(getOption('bDem.PopAgpred', default=FALSE)) {
+			# This is to unblock the GUI, if the run is invoked from bayesDem
+			# and pass info about its status
+			# In such a case the gtk libraries are already loaded
+			gui.options$bDem.PopAgpred.status <- paste('finished', reg.idx, status.for.gui)
+			unblock.gtk('bDem.PopAgpred', gui.options)
+		}
 		id <- regions[reg.idx]
 		if(verbose) cat('\nAggregating region ', id)
 		countries <- get.countries.for.region(id, pop.pred)
@@ -188,6 +234,7 @@ pop.aggregate.independence <- function(pop.pred, regions, name, verbose=verbose)
 		valid.regions[reg.idx] <- TRUE
 		countries.index <- which(is.element(pop.pred$countries[,'code'], countries))
 		for(cidx in 1:length(countries.index)) {
+			country.obs.idx <- grep(paste('^', countries[cidx], '_', sep=''), rownames(obs.data[['male']]), value=FALSE)
 			traj.file <- file.path(pop.pred$output.directory, paste('totpop_country', countries[cidx], '.rda', sep=''))
 			load(traj.file)
 			if(cidx == 1) {
@@ -197,6 +244,10 @@ pop.aggregate.independence <- function(pop.pred, regions, name, verbose=verbose)
 				stotp.hch <- totp.hch
 				stotpm.hch <- totpm.hch
 				stotpf.hch <- totpf.hch
+				aggr.obs.dataM <- obs.data[['male']][country.obs.idx,]
+				aggr.obs.dataF <- obs.data[['female']][country.obs.idx,]
+				rownames(aggr.obs.dataM) <- rownames(aggr.obs.dataF) <- sub(paste(countries[cidx], '_', sep=''), 
+																paste(id, '_', sep=''), rownames(obs.data[['male']][country.obs.idx,]))
 			} else {
 				stotp <- stotp + totp
 				stotpm <- stotpm + totpm
@@ -204,6 +255,8 @@ pop.aggregate.independence <- function(pop.pred, regions, name, verbose=verbose)
 				stotp.hch <- stotp.hch + totp.hch
 				stotpm.hch <- stotpm.hch + totpm.hch
 				stotpf.hch <- stotpf.hch + totpf.hch
+				aggr.obs.dataM <- aggr.obs.dataM + obs.data[['male']][country.obs.idx,]
+				aggr.obs.dataF <- aggr.obs.dataF + obs.data[['female']][country.obs.idx,]
 			}
 		}
 		totp <- stotp
@@ -232,6 +285,8 @@ pop.aggregate.independence <- function(pop.pred, regions, name, verbose=verbose)
 		mean_sdF[id.idx,1,] <- apply(sstotpf, 1, mean, na.rm = TRUE)
 		mean_sdF[id.idx,2,] = apply(sstotpf, 1, sd, na.rm = TRUE)
 		aggregated.countries[[as.character(id)]] <- countries
+		aggr.obs.data[['male']] <- rbind(aggr.obs.data[['male']], aggr.obs.dataM)
+		aggr.obs.data[['female']] <- rbind(aggr.obs.data[['female']], aggr.obs.dataF)
 	}
 	aggr.pred <- pop.pred
 	which.reg.index <- function(x, set) return(which(set == x))
@@ -250,6 +305,7 @@ pop.aggregate.independence <- function(pop.pred, regions, name, verbose=verbose)
 	aggr.pred$traj.mean.sdF <- mean_sdF
 	aggr.pred$aggregation.method <- 'independence'
 	aggr.pred$aggregated.countries <- aggregated.countries
+	aggr.pred$inputs$pop.matrix <- aggr.obs.data
 	bayesPop.prediction <- aggr.pred
 	save(bayesPop.prediction, file=file.path(outdir, 'prediction.rda'))
 	cat('\nAggregations stored into', outdir, '\n')
